'Flash Point RamPage2:  Spin Driver1
'Copyright 2012, Raymond Allen, Rayslogic.com, LLC

'You must define your Pin usage below before use!

'Usage Notes:
'  Both flash and sram 4-bit SQI and because using 2 chips, a 2-byte word is the basic unit of data


CON    'Define Flash Point Pin Usage
  'Data bus
  Sio0Pin=0         'assuming data bus is 8 pins in a row
  Sio3Pin=Sio0Pin+3
  Sio4Pin=Sio0Pin+4
  Sio7Pin=Sio0Pin+7
  'SPI mode pins
  MOSI1=Sio0Pin     'MOSI #1:  Input to flash chip, output of Prop
  MISO1=Sio0Pin+1   'MISO #1:  Output to flash chip, input of Prop
  MOSI2=Sio0Pin+4   'MOSI #2:  Input to flash chip, output of Prop
  MISO2=Sio0Pin+5   'MISO #2:  Output to flash chip, input of Prop
  'Control pins
  ClockPin=8
  FlashEnable=9
  SramEnable=10
 


Var
  word InBuffer[6]   'data buffers are words because one byte from each chip   
  byte CmdBuffer[8]  'commands are bytes because both chips get same commands

PUB start:okay | i  'Start Flash Point Rampage:
    'Apply power,
    'Init Flash (get SPI JEDEC, Set SQI Mode, get SQI JEDEC)
    'Init SRAM (switch to sequential mode)


  'Flash and SRAM chip enable/select need to start high
  outa[FlashEnable]~~
  dira[FlashEnable]~~
  outa[SramEnable]~~
  dira[SramEnable]~~


  'Clocking in all 1's to reset to SPI mode
  outa[0..7]:=-1
  dira[0..7]:=-1
  outa[ClockPin]~~
  dira[ClockPin]~~
  outa[FlashEnable]~
  outa[SramEnable]~
  repeat 10
    outa[ClockPin]~
    outa[ClockPin]~~
  outa[FlashEnable]~~
  outa[SramEnable]~~

  'Read in JEDEC code in SPI mode, should be $BF2601 for both chips
  SpiOutIn(FlashEnable,$9F,8,0,24)
  if (dirb <> $BF2601) or  (outb <> $BF2601)
    return false  'Something is wrong!    
  
  
  'Set Quad I/O mode for flash and SRAM
  SpiOutIn(FlashEnable,$38,8,0,0)
  SpiOutIn(SramEnable,$38,8,0,0) 
  

  'Read in JEDEC code in SQI mode, should be $BF2601
  CmdBuffer[0]:=$AF
  SqiReadWords(FlashEnable,@CmdBuffer,1,0,3,@InBuffer)
  if InBuffer[0]<>$BBFF or  InBuffer[1]<>$2266 or InBuffer[2]<>$0011
    return false  'Something is wrong!     

  
    

  return true 'Successful connection to RP2 flash and ram!


PUB FlashReadWords(pWords,a,n)|i  'read in n words of data from flash (start address a) and store at pWords
  CmdBuffer[0]:=$0B  'High speed read
  dirb:=a /2
  CmdBuffer[1]:=dirb[23..16]
  CmdBuffer[2]:=dirb[15..8]
  CmdBuffer[3]:=dirb[7..0]
  SqiReadWords(FlashEnable,@CmdBuffer[0],4,1,n,pWords)
  
PUB SramReadWords(pWords,a,n)'read n words from SRAM address, a
  dirb:=a/2
  CmdBuffer[0]:=$03 
  CmdBuffer[1]:=dirb[23..16]
  CmdBuffer[2]:=dirb[15..8]
  CmdBuffer[3]:=dirb[7..0]
  SqiReadWords(SramEnable,@CmdBuffer,4,1,n,pWords)

PUB SramWriteWords(address,nWords,pData)  'Write nWords words from hub address, pData, to sram at address
  dirb:=address/2
  CmdBuffer[0]:=$02  'write
  CmdBuffer[1]:=dirb[23..16]
  CmdBuffer[2]:=dirb[15..8]
  CmdBuffer[3]:=dirb[7..0]    
  SqiWriteWords(SramEnable,@CmdBuffer,4,nWords,pData)

PUB FlashWriteWords(address,nWords,pData)|i,n,a  'Write nWords words to RP2 flash at address from given data pointer, pData
  'Note:  writing to flash requires writing an entire page of 256 words.
  'So, the address, a, should be on a page boundary (lower 9 bits all zero)
  'Note:  Entire page must be erased before writing

  'Write Enable
  FlashWriteEnable
  'Program Flash
  a:=0
  repeat while nWords>0
    n:=nWords
    if n>256
      n:=256
    nWords-=n
    FlashSqiPageProgram(a+address,n,pData)
    a+=512
    pData+=512
    'ser.str(String("Programming Flash.  Words Remaining:"))
    'ser.dec(nWords)
    'ser.tx(13)

    'Read status register
    i:=1
    repeat     'Wait for page program to complete by polling status  
      i:=FlashGetStatus   
      'ser.str(String("Status= "))
      'ser.bin(i,16)
      'ser.tx(13)
    until i==0

    'Write Enable
    FlashWriteEnable
  

PUB CopyFlashToSram(FlashReadAddress,SramWriteAddress,nwords)|i,j,k,a,b

  'Note: SRAM has word indexing, so lower 1 bits in given address must be zero

  repeat a from 0 to ((nwords-1)*2) step 2
    FlashReadWords(@InBuffer,FlashReadAddress+a,1) 'Read a word into buffer from flash
    SramWriteWords(SramWriteAddress+a,1,@InBuffer) 'Write a word from buffer to Sram

 
PUB FlashGetStatus  'Returns FPS chip status
  'return FlashSqiOutIn($05,1,0,1)
  CmdBuffer[0]:=$05
  SqiReadWords(FlashEnable,@CmdBuffer,1,0,1,@InBuffer)
  return InBuffer[0]

PUB FlashWriteEnable 'Enable writing to chip, need to do this before ANY write command
  'Write Enable
  CmdBuffer[0]:=$06
  SqiReadWords(FlashEnable,@CmdBuffer,1,0,0,0)  



PUB FlashGetBlockProtectionReg 'returns address to the 6 bytes from Protection Register
  'Read block protection register
  CmdBuffer[0]:=$72
  SqiReadWords(FlashEnable,@CmdBuffer,1,0,6,@InBuffer)
  return @InBuffer


PUB FlashUnprotect|i  'Writes 0's to protection register, allowing writes to entire chip
  'Write block protection register
  FlashWriteEnable
  'Note:  Need to write all 6 bytes!
  CmdBuffer[0]:=$42
  repeat i from 1 to 6
    CmdBuffer[i]:=0
  SqiReadWords(FlashEnable,@CmdBuffer,7,0,0,0)

PUB FlashChipErase
  FlashWriteEnable
  CmdBuffer[0]:=$C7
  SqiReadWords(FlashEnable,@CmdBuffer,1,0,0,0)  



PUB SpiOutIn(EnablePin,cmd,nc,nd,ni)|i 'SPI Protocol I/O:
'  Send "nc" bits of cmd (command) to SPI port, do "nd" dummy cycles, then read in "ni" bits and return them in DIRB/OUTB
  'In SPI mode, will send same command to both chips
  'The lower chip returns data in DIRB and the upper chip returns data in OUTB  

  'Need clock pin as output
  dira[ClockPin]~~
  
  'SoPin is input to Prop, SiPin is output from Prop
  dira[MOSI1]~~
  dira[MISO1]~
  dira[MOSI2]~~
  dira[MISO2]~

  'Start with clock low for mode0
  outa[ClockPin]~
  'Bring CE low to initiate transfer
  outa[EnablePin]~

  'Send data out   
  'Flash chip samples on rising edge of clk, so need data in place before toggling clock 
  dirb:=cmd              'Using dirb to access individual bits
  repeat i from nc-1 to 0  'sending MSB first
    outa[MOSI1]:=dirb[i]  'set data
    outa[MOSI2]:=dirb[i]  'set data
    outa[ClockPin]~~        'toggle clock
    outa[ClockPin]~

  if (ni=>0)  
    repeat nd  'do dummy clocks
      outa[ClockPin]~~        'toggle clock
      outa[ClockPin]~
     
    dirb:=0   'Lower chip returns in dirb
    outb:=0   'upper chip returns in outb    
    'Read Data in
    repeat i from ni-1 to 1
      dirb[i]:=ina[MISO1]  'first output bit is already there
      outb[i]:=ina[MISO2]  'first output bit is already there
      outa[ClockPin]~~        'toggle clock to get more bits
      outa[ClockPin]~
    dirb[0]:=ina[MISO1]  'get the last bit
    outb[0]:=ina[MISO2]  'get the last bit

  'Bring CE high to end transfer
  outa[EnablePin]~~
        
  return 

 

PUB SqiReadWords(EnablePin,pCmd,nc,nd,ni,pIn)|i 'SQI Protocol I/O to Flash or SRAM:
'  EnablePin can be for flash or sram
'  Send "nc" bytes at address pCmd (command) to SQI port, 
'  then do "nd" dummy cycles, 
'  then read in "ni" words to pIn location in HUB memory
'  Note:  unlimited Words either way
  
  'Need clock pin as output
  dira[ClockPin]~~
  
  'All I/O Pins start as output
  dira[Sio7Pin..Sio0Pin]~~
  

  'Start with clock low for mode0
  outa[ClockPin]~
  'Bring CE low to initiate transfer
  outa[EnablePin]~

  
  'Send data out   
  'Flash chip samples on rising edge of clk, so need data in place before toggling clock 

  repeat i from nc-1 to 0  'sending high nibble first
    dirb:=byte[pCmd++]              'Using dirb to access individual bits  
    outa[Sio3Pin..Sio0Pin]:=dirb[7..4]  'set data
    outa[Sio7Pin..Sio4Pin]:=dirb[7..4]  'set data
    outa[ClockPin]~~        'toggle clock
    outa[ClockPin]~
    outa[Sio3Pin..Sio0Pin]:=dirb[3..0]  'set data
    outa[Sio7Pin..Sio4Pin]:=dirb[3..0]  'set data  
    outa[ClockPin]~~        'toggle clock
    if i>0       'Make all pins inputs before going low the last time
      outa[ClockPin]~

  'Now, make all I/O pins inputs and take clock low
  dira[Sio7Pin..Sio0Pin]~
  outa[ClockPin]~ 
    
  repeat nd<<1  'do dummy clocks
    outa[ClockPin]~~        'toggle clock
    outa[ClockPin]~

  if ni>0
    dirb:=0              'Using dirb and outb to access individual bits     
    'Read Data in
    repeat i from ni-1 to 0
      dirb[15..8]:=ina[Sio7Pin..Sio0Pin]  'first output bit is already there
      outa[ClockPin]~~        'toggle clock to get more bits
      outa[ClockPin]~
      dirb[7..0]:=ina[Sio7Pin..Sio0Pin]  'first output bit is already there
      outa[ClockPin]~~        'toggle clock to get more bits
      outa[ClockPin]~
      word[pIn]:=dirb[15..0]
      pIn+=2
    

  'Bring CE high to end transfer
  outa[EnablePin]~~
        
  return dirb

PUB SqiWriteWords(EnablePin,pCmd,nc,no,pOut)|i 'SQI Protocol I/O:
'  Send "nc" bytes at address pCmd (command) to SPI port,
'  then write in "no" words from pOut
  'SqiOutInBuffer(FlashEnable,pCmd,nc,no,pOut,0,0,0)

  'Need clock pin as output
  dira[ClockPin]~~
  
  'All I/O Pins start as output
  dira[Sio7Pin..Sio0Pin]~~
  

  'Start with clock low for mode0
  outa[ClockPin]~
  'Bring CE low to initiate transfer
  outa[EnablePin]~

  
  'Send data out   
  'Flash chip samples on rising edge of clk, so need data in place before toggling clock 

  repeat i from nc-1 to 0  'sending high nibble first
    dirb:=byte[pCmd++]              'Using dirb to access individual bits  
    outa[Sio3Pin..Sio0Pin]:=dirb[7..4]  'set data
    outa[Sio7Pin..Sio4Pin]:=dirb[7..4]  'set data
    outa[ClockPin]~~        'toggle clock
    outa[ClockPin]~
    outa[Sio3Pin..Sio0Pin]:=dirb[3..0]  'set data
    outa[Sio7Pin..Sio4Pin]:=dirb[3..0]  'set data  
    outa[ClockPin]~~        'toggle clock
    outa[ClockPin]~


  if no==0 'Exit now if not writing words
    outa[EnablePin]~~
    dira[Sio7Pin..Sio0Pin]~  'tristate bus 
    return


  dirb:=0              'Using dirb and outb to access individual bits     
  'Write Data Out
  repeat i from no-1 to 0
    dirb:=word[pOut]
    outa[Sio7Pin..Sio0Pin]:=dirb[15..8]  '
    outa[ClockPin]~~       
    outa[ClockPin]~
    outa[Sio7Pin..Sio0Pin]:=dirb[7..0]  'first output bit is already there
    outa[ClockPin]~~        'toggle clock to get more bits
    outa[ClockPin]~
    'word[pIn]:=dirb[15..0]
    pOut+=2
    

  'Bring CE high to end transfer
  outa[EnablePin]~~
  dira[Sio7Pin..Sio0Pin]~  'tristate bus
        
  return dirb



PUB FlashSqiPageProgram(a,n,p)|i,pCmd,nc 'SQI Protocol I/O: write page of "n" words from Prop address p to Flash address a
  'Assuming lower 9 bits of a are zero, up to 256 words can be programmed

  dirb:=a/2
  CmdBuffer[0]:=$02 'Page Program command
  CmdBuffer[1]:=dirb[23..16]
  CmdBuffer[2]:=dirb[15..8]
  CmdBuffer[3]:=dirb[7..0]
  nc:=4
  pCmd:=@CmdBuffer[0]

  SqiWriteWords(FlashEnable,pCmd,nc,n,p)

  return
  
  'Send data out   
  'Flash chip samples on rising edge of clk, so need data in place before toggling clock 

  repeat i from nc-1 to 0  'sending high nibble first
    dirb:=byte[pCmd++]              'Using dirb to access individual bits  
    outa[Sio3Pin..Sio0Pin]:=dirb[7..4]  'set data
    outa[ClockPin]~~        'toggle clock
    outa[ClockPin]~
    outa[Sio3Pin..Sio0Pin]:=dirb[3..0]  'set data
    outa[ClockPin]~~        'toggle clock
    outa[ClockPin]~

  'Now clock out data    
  repeat i from 0 to n-1  'sending high nibble first
    dirb:=byte[p+i]              'Using dirb to access individual bits  
    outa[Sio3Pin..Sio0Pin]:=dirb[7..4]  'set data
    outa[ClockPin]~~        'toggle clock
    outa[ClockPin]~
    outa[Sio3Pin..Sio0Pin]:=dirb[3..0]  'set data
    outa[ClockPin]~~        'toggle clock
    outa[ClockPin]~
    
  'Bring CE high to end transfer
  outa[FlashEnable]~~

  'Play it safe and return all I/O pins to inputs
  dira[Sio3Pin..Sio0Pin]~
        
  return dirb 


