'Test of RamPage2 SD, Flash and SRAM interfaces
'Copyright (C) 2012-2013 Rayslogic.com, LLC

CON 'Crystal settings
  _clkmode = xtal1+PLL16X
  _xinfreq = 5_000_000

OBJ
  sd: "fsrw"     'SD card driver
  ser: "fullduplexserial"  'serial driver
  rp2: "RP2_SpinDriver1b"  'RamPage2 driver

CON 'SD card pin settings (edit the rp2 driver object for flash and sram pins)
  SD_Do=12
  SD_Clk=13
  SD_Di=14
  SD_Cs=15

CON   'base addresses of flash and sram that we will test write to and read from
  FlashWriteAddress = $000000   'Starting address on Flash to write file to (lower 9 bits should be 0 to align with page boundary)
  SramWriteAddress  = $000000   'Starting address on SRAM to copy file to (from flash)
 
DAT  'Some test data to transfer to flash and SRAM and then read back

FileStart word
word $0123
word $0567
word $09AB
word $0DEF
word $1123
word $1567
word $19AB
word $1DEF
word $2123
word $2567
word $29AB
word $2DEF
word $3123
word $3567
word $39AB
word $3DEF
long $ADCB1234[500]  'test data array
FileEnd byte 0

TestReadFlash word $F9 [5]  'We'll try reading to here when we're done to see it it worked
TestReadSram word $F9 [5]  'We'll try reading to here when we're done to see it it worked

PUB Test_RP2|i, j, k   'Test all basic functions of SRAM and Flash give status over serial connection  
  ser.start(31,30,0,9600)
  repeat i from 100 to 0
    ser.dec(i)
    ser.tx(13)
    waitcnt(cnt+clkfreq/50)

  Test_SD

  'Initialize RP2
  ser.str(string("Starting up RamPage2",13))
  repeat 
    i:=RP2.Start
    if (i==false)
      ser.str(String(13,"Flash Initialization Failed.",13))  'Make sure you specified your pins correctly!
      waitcnt(cnt+clkfreq/2) 
    else
      ser.str(String(13,"Flash Initialization Complete.",13))
  until i==true

  Test_Flash
  Test_SRAM
  Test_Longs 
  
  repeat 'all done
  
PUB Test_Flash  |i,j,k



  'Enable writing to Flash
  RP2.FlashUnprotect

  'For fun, look at Flash block protection register (should be all 0's now)
  j:=RP2.FlashGetBlockProtectionReg
  ser.str(String("Protection Status= "))
  k:=0
  repeat i from 0 to 5
    ser.hex(word[j][i],4)
    k+=word[j][i]
  ser.tx(13)
  if k==0
    ser.str(String("Flash now unprotected from writing.",13))
  else
    ser.str(String("Flash unprotect failure.",13))
    repeat
  
  waitcnt(cnt+clkfreq/2)



  'Erase Entire Flash  'NOTE:  Skip this step and change destination address below to put more files on flash
  ser.str(String("Erasing Flash",13))
  RP2.FlashChipErase
  repeat   'Wait for erase to complete by polling status    
    i:=RP2.FlashGetStatus             
    ser.str(String("Status= "))
    ser.bin(i,16)
    ser.tx(13)
    waitcnt(cnt+clkfreq/10)
  until i==0                   'if status won't go to zero, make sure chip has good power 

  'Write data to Flash
  ser.str(String("Writing to Flash",13)) 
  i:=@FileEnd-@FileStart+1 'This is the number of bytes to write
  RP2.FlashWriteWords(FlashWriteAddress,(i+1)/2,@FileStart)  'Writing to address FlashWriteAddress 
  ser.str(String(13,"Copy file to Flash complete.",13))

  'test flash
  ser.str(String(13,"Test read from flash.",13))
  repeat 1
    repeat i from 0 to 8 step 2
      RP2.FlashReadWords(@TestReadFlash,i,1)
      waitcnt(cnt+clkfreq/2)  
      ser.str(String("Flash: "))
      ser.hex(word[@TestReadFlash],4)
      ser.str(String("  HUB: ")) 
      ser.hex(word[@FileStart+i],4)
      ser.tx(13) 

  ser.str(String(13,"Test read complete.",13,13))


PUB Test_SRAM  |p,i,offset,j
  
  'Transfer the file from Flash to SRAM
  i:=@FileEnd-@FileStart+1 'This is the number of bytes written
  i:=(i+1)/2  'RAM works in words, this turns i bytes into i words (making sure not to skip the end)...
  ser.str(String(13,"Copying Flash to SRAM (please wait)...",13))   
  RP2.CopyFlashToSram(FlashWriteAddress,SramWriteAddress,i)

  'For fun, we'll read back in several words and see if they match original data  
  repeat  1
    repeat offset from 0 to 64 step 2 
     ser.str(String("Testing Data at offset: "))
     ser.dec(offset)
    'read in 2 bytes flash (start address FlashWriteAddress+offset) and store at TestRead 
     RP2.FlashReadWords(@TestReadFlash,FlashWriteAddress+offset,1)      
     'read first 2 bytes from SRAM too
     RP2.SramReadWords(@TestReadSram,SramWriteAddress+offset,1)
     
     'all done, just keep showing result of read and original
     ser.str(String(13,"Two bytes of Embedded file:  "))
     'repeat i from 0 to 3
       ser.hex(word[@FileStart+offset],4)
     ser.str(String(13,"Two bytes read from RP2 Flash:     "))
     'repeat i from 0 to 3
       ser.hex(word[@TestReadFlash],4)
     ser.str(String(13,"Two bytes read from RP2 SRAM:     "))
     ser.hex(word[@TestReadFlash],4)      
     waitcnt(cnt+clkfreq)
     ser.tx(13)
     ser.tx(13)
           

PUB Test_Longs|i,j,k    'Test writing and reading long values from SRAM
  i:=$A29342  'some number
  repeat
    i++
    ser.str(String(13,"Writing this long to SRAM: $")) 
    ser.hex(i,8) 'print a long
    RP2.SramWriteWords(0,2,@i) 'write long to sram
    RP2.SramReadWords(@j,0,2) 'read long from sram
    ser.str(String(13,"Read this long from SRAM: $")) 
    ser.hex(i,8) 'print a long
    ser.tx(13) 
    waitcnt(cnt+clkfreq)  





    
PUB Test_SD|i,j,k     'list contents of SD card to make sure it works

  'See if SD inserted
  DIRA[11]~
  repeat
    if INA[11]==1
      ser.str(string("SD card detected."))
      quit
    ser.str(string("Insert SD card.",13))
    waitcnt(cnt+clkfreq/4)

  
  'Turn on LED
  waitcnt(cnt+clkfreq)
  dira[11]~~
  outa[11]~~  
       

  repeat   'try a few times to mount the SD card
    i:=sd.mount_explicit(SD_DO, SD_CLK, SD_DI, SD_CS)
    if i==0
       quit
    ser.str(string("SD mount failed"))
    waitcnt(cnt+clkfreq/4)

  ser.str(string("SD mount success",13))
  waitcnt(cnt+clkfreq)
  
  sd.opendir
  repeat
    i:=sd.nextfile(@fname)
    if i<0
      quit
    ser.str(@fname)
    ser.tx(13)

  ser.str(string("No more files",13))
  waitcnt(cnt+clkfreq) 
  return


DAT
fname   byte "i"[12],0
  
   
                        